%define maj     81.3
%define libname     %mklibname quictls %{maj}
%define develname   %mklibname quictls -d
%define staticname  %mklibname quictls -s -d

%define sslname       openssl-quic
%define ssllibname    %mklibname openssl-quic 3
%define ssldevelname  %mklibname openssl-quic -d
%define sslstaticname %mklibname openssl-quic -s -d

Summary:    Secure Sockets Layer communications libs & utils
Name:       quictls
Version:    3.0.17
#% define     subrel 1
Release:    %mkrel 1
License:    ASL 2.0
Group:      System/Libraries
URL:        https://www.openssl.org/
Source0:    https://github.com/quictls/openssl/archive/refs/tags/openssl-3.0.15-quic1.tar.gz
Source2:    Makefile.certificate
Source3:    genpatches
Source4:    openssl-thread-test.c
Source6:    make-dummy-cert
Source7:    renew-dummy-cert
Source9:    configuration-switch.h            
Source10:   configuration-prefix.h
Source12:   ec_curve.c
Source13:   ectest.c

# fedora patches
# Patches exported from source git
# Aarch64 and ppc64le use lib64
Patch1: 0001-Aarch64-and-ppc64le-use-lib64.patch
# Use more general default values in openssl.cnf
Patch2: 0002-Use-more-general-default-values-in-openssl.cnf.patch
# Do not install html docs
Patch3: 0003-Do-not-install-html-docs.patch
# Override default paths for the CA directory tree
Patch4: 0004-Override-default-paths-for-the-CA-directory-tree.patch
# apps/ca: fix md option help text
Patch5: 0005-apps-ca-fix-md-option-help-text.patch
# Disable signature verification with totally unsafe hash algorithms
Patch6: 0006-Disable-signature-verification-with-totally-unsafe-h.patch
# Add support for PROFILE=SYSTEM system default cipherlist
Patch7: 0007-Add-support-for-PROFILE-SYSTEM-system-default-cipher.patch
# Add FIPS_mode() compatibility macro
Patch8: 0008-Add-FIPS_mode-compatibility-macro.patch
# Add check to see if fips flag is enabled in kernel
#Patch9: 0009-Add-Kernel-FIPS-mode-flag-support.patch
# remove unsupported EC curves
Patch11: 0011-Remove-EC-curves.patch
# Instructions to load legacy provider in openssl.cnf
Patch24: 0024-load-legacy-prov.patch 
# Selectively disallow SHA1 signatures rhbz#2070977
Patch49: 0049-Allow-disabling-of-SHA1-signatures.patch
# Backport of patch for RHEL for Edge rhbz #2027261
Patch51: 0051-Support-different-R_BITS-lengths-for-KBKDF.patch
# Support SHA1 in TLS in LEGACY crypto-policy (which is SECLEVEL=1)
Patch52: 0052-Allow-SHA1-in-seclevel-1-if-rh-allow-sha1-signatures.patch

# Security patches
Patch53: openssl-pkcs1-implicit-rejection.patch
Patch54: openssl-3.0.15-to-3.0.16.patch
Patch55: openssl-3.0.16-to-3.0.17.patch

# fix library names as 'variant' so to not interfere with main openssl
Patch100: 0100-openssl-3.0.5+quic-change-soname.patch
Patch101: 0101-openssl-3.0.12+quic-change-pkgconfig-pc.patch
Patch102: 0102-openssl-3.0.12+quic-change-includedir-openssl.patch
Patch103: 0103-openssl-3.0.5+quic-change-ossl-modules-dir.patch
Patch104: 0104-openssl-3.0.5+quic-change-staticname.patch
Patch105: 0105-openssl-3.0.5+quic-honor-shlib_variant.patch

Requires:   %{libname} = %{version}-%{release}
Requires:   openssl
Requires:   rootcerts
BuildRequires:  multiarch-utils >= 1.0.3
BuildRequires:  chrpath
BuildRequires:  pkgconfig(zlib)
BuildRequires:  pkgconfig(libsctp)
BuildRequires:  git
%ifarch %{arm}
BuildRequires:  libatomic-devel
%endif
# (tv) for test suite:
BuildRequires:  bc

%description
The openssl certificate management tool and the shared libraries that provide
various encryption and decryption algorithms and protocols, including DES, RC4,
RSA and SSL.

%package -n %{libname}
Summary:    Secure Sockets Layer communications libs
Group:      System/Libraries
Requires:   crypto-policies
Provides:   %{libname} = %{version}-%{release}
Provides:   %{ssllibname} = %{version}-%{release}

%description -n %{libname}
The libraries files are needed for various cryptographic algorithms
and protocols, including DES, RC4, RSA and SSL.

%package -n %{develname}
Summary:    Secure Sockets Layer communications libs & headers & utils
Group:      Development/Other
Requires:   %{libname} = %{version}-%{release}
Provides:   %{develname} = %{version}-%{release}
Provides:   %{name}-devel = %{version}-%{release}
Provides:   lib%{name}-devel = %{version}-%{release}
Provides:   %{sslname}-devel = %{version}-%{release}
Provides:   %{ssldevelname} = %{version}-%{release}

%description -n %{develname}
The libraries and include files needed to compile apps with support
for various cryptographic algorithms and protocols, including DES, RC4, RSA
and SSL.

%package -n %{staticname}
Summary:    Secure Sockets Layer communications static libs
Group:      Development/Other
Requires:   %{libname} = %{version}-%{release}
Provides:   %{staticname} = %{version}-%{release}
Provides:   %{name}-static-devel = %{version}-%{release}
Provides:   lib%{name}-static-devel = %{version}-%{release}
Provides:   %{sslname}-static-devel = %{version}-%{release}
Provides:   %{sslstaticname} = %{version}-%{release}

%description -n %{staticname}
The static libraries needed to compile apps with support for various
cryptographic algorithms and protocols, including DES, RC4, RSA and SSL.

%package    perl
Summary:    Perl scripts provided with OpenSSL
Group:      System/Libraries
Requires:   %{name} = %{version}-%{release}
Conflicts:  %name <= 1.0.2h-1.mga6

%description    perl
OpenSSL is a toolkit for supporting cryptography. The openssl-perl
package provides Perl scripts for converting certificates and keys
from other formats to the formats used by the OpenSSL toolkit.

%prep
%autosetup -S git -n openssl-openssl-3.0.15-quic1

cp %{SOURCE12} crypto/ec/
cp %{SOURCE13} test/

%build
%serverbuild

# Figure out which flags we want to use.
# default
sslarch=%{_os}-%{_target_cpu}
%ifarch %ix86
sslarch=linux-elf
if ! echo %{_target} | grep -q i[56]86 ; then
    sslflags="no-asm 386"
fi
%endif
%ifarch x86_64
sslflags=enable-ec_nistp_64_gcc_128
%endif
%ifarch %{arm}
sslarch=linux-armv4
%endif

# Add -Wa,--noexecstack here so that libcrypto's assembler modules will be
# marked as not requiring an executable stack.
# Also add -DPURIFY to make using valgrind with openssl easier as we do not
# want to depend on the uninitialized memory as a source of entropy anyway.
RPM_OPT_FLAGS="$RPM_OPT_FLAGS -Wa,--noexecstack -Wa,--generate-missing-build-notes=yes -DPURIFY $RPM_LD_FLAGS"

export HASHBANGPERL=/usr/bin/perl

# ia64, x86_64, ppc, ppc64 are OK by default
# Configure the build tree.  Override OpenSSL defaults with known-good defaults
# usable on all platforms.  The Configure script already knows to use -fPIC and
# RPM_OPT_FLAGS, so we can skip specifiying them here.
./Configure \
    --prefix=%{_prefix} --openssldir=%{_sysconfdir}/pki/tls-quic ${sslflags} \
    --system-ciphers-file=%{_sysconfdir}/crypto-policies/back-ends/openssl.config \
     zlib enable-camellia enable-seed enable-rfc3779 enable-sctp \
     enable-cms enable-md2 enable-rc5 enable-ktls enable-fips \
     no-mdc2 no-ec2m no-sm2 no-sm4 \
     shared ${sslarch} $RPM_OPT_FLAGS '-DDEVRANDOM="\"/dev/urandom\""'

%make_build all

# Clean up the .pc files
for i in libcrypto-quic.pc libssl-quic.pc openssl-quic.pc ; do
    sed -i '/^Libs.private:/{s/-L[^ ]* //;s/-Wl[^ ]* //}' $i
done

%check

(sysctl net.sctp.addip_enable=1 && sysctl net.sctp.auth_enable=1) || \
(echo 'Failed to enable SCTP AUTH chunks, disabling SCTP for tests...' &&
    sed '/"msan" => "default",/a\ \ "sctp" => "default",' configdata.pm > configdata.pm.new && \
    touch -r configdata.pm configdata.pm.new && \
    mv -f configdata.pm.new configdata.pm)

# We must revert patch4 before tests otherwise they will fail
patch -p1 -R < %{PATCH4}

export OPENSSL_ENABLE_MD5_VERIFY=
export OPENSSL_SYSTEM_CIPHERS_OVERRIDE=xyz_nonexistent_file

make test

%install
%make_install

# make the rootcerts dir
install -d %{buildroot}%{_sysconfdir}/pki/tls-quic/rootcerts

# Install a makefile for generating keys and self-signed certs, and a script
# for generating them on the fly.
mkdir -p %{buildroot}%{_sysconfdir}/pki/tls-quic/certs
install -m 644 %{SOURCE2} %{buildroot}%{_sysconfdir}/pki/tls-quic/certs/Makefile

# Install rootcerts symlinks
ln -s ../../../etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem %{buildroot}%{_sysconfdir}/pki/tls-quic/cert.pem
ln -s ../../../../etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem %{buildroot}%{_sysconfdir}/pki/tls-quic/certs/ca-bundle.crt
ln -s ../../../../etc/pki/ca-trust/extracted/openssl/ca-bundle.trust.crt %{buildroot}%{_sysconfdir}/pki/tls-quic/certs/ca-bundle.trust.crt

# Move runable perl scripts to bindir
mv %{buildroot}%{_sysconfdir}/pki/tls-quic/misc/*.pl %{buildroot}%{_bindir}
mv %{buildroot}%{_sysconfdir}/pki/tls-quic/misc/tsget %{buildroot}%{_bindir}

# Rename man pages so that they don't conflict with other system man pages.
pushd %{buildroot}%{_mandir}
mv man5/config.5ossl man5/openssl.cnf.5
popd

install -d %{buildroot}%{_sysconfdir}/pki/CA
install -d %{buildroot}%{_sysconfdir}/pki/CA/private
install -d %{buildroot}%{_sysconfdir}/pki/CA/certs
install -d %{buildroot}%{_sysconfdir}/pki/CA/crl
install -d %{buildroot}%{_sysconfdir}/pki/CA/newcerts

rm -f %{buildroot}%{_sysconfdir}/pki/tls-quic/openssl.cnf.dist
rm -f %{buildroot}%{_sysconfdir}/pki/tls-quic/ct_log_list.cnf.dist
%ifarch i686
rm -f %{buildroot}%{_sysconfdir}/pki/tls-quic/fipsmodule.cnf
%endif

%multiarch_includes %{buildroot}%{_includedir}/openssl-quic/openssl/opensslconf.h

# nuke rpath
chrpath -d %{buildroot}%{_bindir}/openssl
mv %{buildroot}%{_bindir}/openssl %{buildroot}%{_bindir}/quictls

# Fix libdir.
for i in %{buildroot}%{_libdir}/pkgconfig/*.pc; do
    sed -i 's,^libdir=${exec_prefix}/lib$,libdir=${exec_prefix}/%{_lib},g' $i
done

# adjust ssldir
perl -pi -e "s|^\\\$CATOP\=\".*|\\\$CATOP\=\"%{_sysconfdir}/pki/tls-quic\";|g" %{buildroot}%{_bindir}/CA.pl
perl -pi -e "s|\./demoCA|%{_sysconfdir}/pki/tls-quic|g" %{buildroot}%{_sysconfdir}/pki/tls-quic/openssl.cnf

%files
%doc LICENSE.txt NEWS.md README.md
%dir %{_sysconfdir}/pki
%dir %{_sysconfdir}/pki/tls-quic
%dir %{_sysconfdir}/pki/tls-quic/certs
%dir %{_sysconfdir}/pki/tls-quic/misc
%dir %{_sysconfdir}/pki/tls-quic/private
%dir %{_sysconfdir}/pki/tls-quic/rootcerts
%config(noreplace) %{_sysconfdir}/pki/tls-quic/openssl.cnf
%config(noreplace) %{_sysconfdir}/pki/tls-quic/ct_log_list.cnf
%ifnarch i686
%config(noreplace) %{_sysconfdir}/pki/tls-quic/fipsmodule.cnf
%endif
%{_sysconfdir}/pki/tls-quic/cert.pem
%{_sysconfdir}/pki/tls-quic/certs/ca-bundle.crt
%{_sysconfdir}/pki/tls-quic/certs/ca-bundle.trust.crt
%{_sysconfdir}/pki/tls-quic/certs/Makefile
%{_bindir}/quictls
%exclude %{_mandir}/man[157]/*
%exclude %{_mandir}/man1*/*rehash*
%exclude %{_mandir}/man1*/*.pl*
%exclude %{_mandir}/man1*/*tsget*

%files -n %{libname}
%doc LICENSE.txt
%{_libdir}/lib*.so.%{maj}
%{_libdir}/engines-quic-%{maj}
%{_libdir}/ossl-quic-modules

%files -n %{develname}
%doc CHANGES.md doc/dir-locals.example.el doc/openssl-c-indent.el
%dir %{_includedir}/openssl-quic/openssl
%multiarch %{multiarch_includedir}/openssl-quic/openssl/opensslconf.h
%{_includedir}/openssl-quic/openssl/*.h
%{_libdir}/lib*.so
%{_mandir}/man3/*
%{_libdir}/pkgconfig/*.pc

%files -n %{staticname}
%{_libdir}/lib*.a

%files perl
%{_bindir}/c_rehash
%{_bindir}/*.pl
%{_bindir}/tsget
%{_mandir}/man1*/*rehash*
%{_mandir}/man1*/*.pl*
%{_mandir}/man1*/*tsget*
%dir %{_sysconfdir}/pki/CA
%dir %{_sysconfdir}/pki/CA/private
%dir %{_sysconfdir}/pki/CA/certs
%dir %{_sysconfdir}/pki/CA/crl
%dir %{_sysconfdir}/pki/CA/newcerts


%changelog
* Thu Jul 17 2025 rapsys <rapsys> 3.0.17-1.mga9
+ Revision: 2256786
- Version 3.0.17
- Fixes to get it applied on top of quictls-3.0.15 tree
- Add openssl update patches
- add upstream patch for CVE-2024-13176 (mga#33942)
- port a patch from Ubuntu to allow fixing CVE-2023-50782 in python-cryptography (mga#32584) SOURCES/openssl-pkcs1-implicit-rejection.patch
- add upstream patch for CVE-2024-13176 (mga#33942)
- Add upstream patch for CVE-2024-9143
- Version 3.0.15 final
- New version 3.0.15 for CVE-2024-6119
- New version 3.0.15 for CVE-2024-6119
- New version 3.0.15 for CVE-2024-6119
- New version 3.0.15 for CVE-2024-6119
- Upstream 3.0.14 quic1 release
- Upstream 3.0.14 quic1 release
- Version 3.0.14
  Sync with openssl patch set
- add upstream patches for CVE-2023-5678, CVE-2023-6129, CVE-2023-6237 and CVE-2024-0727 (mga#32498)
- Version 3.0.13
- Version 3.0.13
- Version 3.0.13
- Fix subrel position
- add upstream patches for CVE-2023-5678, CVE-2023-6129, CVE-2023-6237 and CVE-2024-0727 (mga#32794)
- Fix patches to build
- new version 3.0.12 for CVE-2023-5363 (mga#32484)
- Version 3.0.10

* Mon Jun 12 2023 rapsys <rapsys> 3.0.9-1.mga9
+ Revision: 1961840
- Sync version and patchset with openssl 3.0.9-1

* Wed Apr 26 2023 rapsys <rapsys> 3.0.8-1.mga9
+ Revision: 1954983
- Sync with openssl from r1944056 to r1954982
  new version 3.0.8 for CVE-2022-4203, CVE-2022-4304, CVE-2022-4450, CVE-2023-021[5-7], CVE-2023-0286, CVE-2023-0401 (mga#31526)
  backport upstream patches for CVE-2023-046[4-6] (mga#31526)
  backport an upstream patch for CVE-2023-1255
- Sync patchset with openssl rev 1934473

* Sun Dec 04 2022 rapsys <rapsys> 3.0.5-7.mga9
+ Revision: 1917155
- Rename tls to tls-quic config dir to prevent conflict
- Fix quicktls conflict

* Sat Dec 03 2022 rapsys <rapsys> 3.0.5-5.mga9
+ Revision: 1916131
- Fix devel and static-devel requires and provides

* Sat Dec 03 2022 rapsys <rapsys> 3.0.5-4.mga9
+ Revision: 1915886
- Revert patch 0024
  Sync with openssl patchset
  Add bug #31066 patchset to avoid conflicts with openssl

* Fri Nov 04 2022 rapsys <rapsys> 3.0.5-3.mga9
+ Revision: 1902512
- Remove conflicting fips and legacy ossl modules

* Fri Nov 04 2022 rapsys <rapsys> 3.0.5-2.mga9
+ Revision: 1902445
- Sync patchset with openssl-3.0.5:
- add upstream patch to fix CVE-2022-3358
- add patch from fedora to fix CVE-2022-3602 and CVE-2022-3786

* Mon Oct 24 2022 rapsys <rapsys> 3.0.5-1.mga9
+ Revision: 1898918
- Import quictls 3.0.5 with existing patches from openssl 3.0.5
- Created package structure for quictls.

